from .User import User


class TextMsg:
    def __init__(self, *,
                 channel_type: str,
                 target_id: str, author_id: str, content: str, msg_id: str,
                 msg_timestamp: int, nonce: str, extra):
        self.channel_type = channel_type
        self.type = 1
        self.target_id = target_id
        self.author_id = author_id
        self.content = content
        self.msg_id = msg_id
        self.msg_timestamp = msg_timestamp
        self.nonce = nonce

        self.guild_id: str = extra['guild_id']
        self.channel_name: str = 'channel_name' in extra.keys() and extra['channel_name'] or ''
        self.mention: list = extra['mention']
        self.mention_all: bool = extra['mention_all']
        self.mention_roles: list = extra['mention_roles']
        self.mention_here: bool = extra['mention_here']
        self.author: User = User(extra['author'])




import asyncio


class Command:
    def __init__(self, func, name: str):
        if not asyncio.iscoroutinefunction(func):
            raise TypeError('handler must be a coroutine.')
        self.handler = func

        self.name = name = name or func.__name__
        if not isinstance(name, str):
            raise TypeError('Name of a command must be a string.')

    @staticmethod
    def command(name: str):
        def decorator(func):
            return Command(func, name)

        return decorator



import base64

from Cryptodome.Cipher import AES
from Cryptodome.Util import Padding


class Cert:
    def __init__(self, *,
                 client_id: str, client_secret: str,
                 token: str, verify_token: str,
                 encrypt_key: str = ''):
        self.client_id = client_id
        self.client_secret = client_secret
        self.token = token
        self.verify_token = verify_token
        self.encrypt_key = encrypt_key

    def decrypt(self, data: bytes) -> str:
        if not self.encrypt_key:
            return ''
        data = base64.b64decode(data)
        data = AES.new(key=self.encrypt_key.encode('utf-8').ljust(32, b'\x00'),
                       mode=AES.MODE_CBC, iv=data[0:16]).decrypt(base64.b64decode(data[16:]))
        data = Padding.unpad(data, 16)
        return data.decode('utf-8')



import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="khl.py",
    version="0.0.5",
    author="TWT233",
    author_email="TWT2333@outlook.com",
    description="SDK for kaiheila.cn in python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TWT233/khl.py",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.4',
)



import asyncio
import json
import shlex
import zlib
from inspect import signature, Parameter
from typing import Union

from aiohttp import web, ClientSession

from .cert import Cert
from ..utils import API_URL, TextMsg, Command


class Bot:
    def __init__(self, *,
                 port: int = 5000, compress: bool = True,
                 cmd_prefix: Union[list, str, tuple] = ('!', '！'),
                 cert: Cert):
        self.port: int = port
        self.compress = compress
        self.cmd_prefix = [i for i in cmd_prefix]
        self.cert = cert

        self.app = web.Application()
        self.__cmd_list: dict = {}

    def add_command(self, cmd: Command):
        if not isinstance(cmd, Command):
            raise TypeError('not a Command')
        if cmd.name in self.__cmd_list.keys():
            raise ValueError('Command Name Exists')
        self.__cmd_list[cmd.name] = cmd

    def command(self, name: str):
        def decorator(func):
            cmd = Command.command(name)(func)
            self.add_command(cmd)

        return decorator

    def split_msg_args(self, msg: TextMsg):
        return (msg.content[0] not in self.cmd_prefix) and None or shlex.split(msg.content[1:])

    async def __msg_handler(self, msg: TextMsg):
        arg_list = self.split_msg_args(msg)
        if arg_list:
            if arg_list[0] in self.__cmd_list.keys():
                func = self.__cmd_list[arg_list[0]].handler
                argc = len([1 for v in signature(func).parameters.values() if v.default == Parameter.empty])
                if argc <= len(arg_list):
                    await func(msg, *arg_list[1:len(signature(func).parameters)])

    def data_to_json(self, data: bytes):
        data = self.compress and zlib.decompress(data) or data
        data = json.loads(str(data, encoding='utf-8'))
        return ('encrypt' in data.keys()) and json.loads(self.cert.decrypt(data['encrypt'])) or data

    async def send(self, channel_id: str, content: str, *, quote: str = '', object_name: int = 1, nonce: str = ''):
        headers = {'Authorization': f'Bot {self.cert.token}', 'Content-type': 'application/json'}
        data = {'channel_id': channel_id, 'content': content, 'object_name': object_name, 'quote': quote,
                'nonce': nonce}
        return await ClientSession().post(f'{API_URL}/channel/message?compress=0', headers=headers, json=data)

    def run(self):
        async def respond(request: web.Request) -> web.Response:
            json_data = self.data_to_json(await request.read())
            assert json_data
            assert json_data['d']['verify_token'] == self.cert.verify_token

            if json_data['s'] == 0:
                d = json_data['d']
                if d['type'] == 1:
                    msg = TextMsg(channel_type=d['channel_type'], target_id=d['target_id'],
                                  author_id=d['author_id'], content=d['content'], msg_id=d['msg_id'],
                                  msg_timestamp=d['msg_timestamp'], nonce=d['nonce'], extra=d['extra'])
                    asyncio.ensure_future(self.__msg_handler(msg))
                if d['type'] == 255:
                    if d['channel_type'] == 'WEBHOOK_CHALLENGE':
                        return web.json_response({'challenge': d['challenge']})

            return web.Response(status=200)

        self.app.router.add_post('/khl-wh', respond)
        web.run_app(self.app, port=self.port)
